/*
 * Project: MoreOptions
 * File: MoreOptions.prg
 * Description: Editor extra options
 * Author: Jose Lalin
 * Date:
 */

#include "Xailer.ch"


DYNAMIC TIDEPlugin

#define SCWS_INVISIBLE           0
#define SCWS_VISIBLEALWAYS       1
#define SCWS_VISIBLEAFTERINDENT  2

#define SC_EOL_CRLF        0
#define SC_EOL_CR          1
#define SC_EOL_LF          2
#define SCI_CONVERTEOLS    2029

#define SCI_SETCARETWIDTH 2188
#define SCI_GETCARETWIDTH 2189

#define SCI_STYLESETBACK 2052
#define SCI_STYLEGETBACK 2482

#define SCI_GETCARETSTYLE 2513
#define SCI_SETMULTIPLESELECTION 2563
#define SCI_GETMULTIPLESELECTION 2564
#define SCI_SETADDITIONALSELECTIONTYPING 2565
#define SCI_GETADDITIONALSELECTIONTYPING 2566

#define SCI_SETMULTIPASTE 2614
#define SCI_GETMULTIPASTE 2615

#define STYLE_BRACELIGHT 34

#define SCI_SETEXTRAASCENT 2525
#define SCI_GETEXTRAASCENT 2526
#define SCI_SETEXTRADESCENT 2527
#define SCI_GETEXTRADESCENT 2528

#define SC_WRAP_NONE 0
#define SC_WRAP_WORD 1
#define SC_WRAP_CHAR 2
#define SC_WRAP_WHITESPACE 3
#define SCI_SETWRAPMODE 2268
#define SCI_GETWRAPMODE 2269
#define SC_WRAPVISUALFLAG_NONE 0x0000
#define SC_WRAPVISUALFLAG_END 0x0001
#define SC_WRAPVISUALFLAG_START 0x0002
#define SC_WRAPVISUALFLAG_MARGIN 0x0004
#define SCI_SETWRAPVISUALFLAGS 2460
#define SCI_GETWRAPVISUALFLAGS 2461
#define SC_WRAPVISUALFLAGLOC_DEFAULT 0x0000
#define SC_WRAPVISUALFLAGLOC_END_BY_TEXT 0x0001
#define SC_WRAPVISUALFLAGLOC_START_BY_TEXT 0x0002
#define SCI_SETWRAPVISUALFLAGSLOCATION 2462
#define SCI_GETWRAPVISUALFLAGSLOCATION 2463
#define SCI_SETWRAPSTARTINDENT 2464
#define SCI_GETWRAPSTARTINDENT 2465
#define SC_WRAPINDENT_FIXED 0
#define SC_WRAPINDENT_SAME 1
#define SC_WRAPINDENT_INDENT 2
#define SCI_SETWRAPINDENTMODE 2472
#define SCI_GETWRAPINDENTMODE 2473


//------------------------------------------------------------------------------

INIT PROC MoreOptionsRegister()

   WITH OBJECT IMoreOptions():New()
      :cName := "MoreOptions"
      :cDescription := "Editor extra options"
      :cVersion := "1.5"
      :cCategory := "Editor"
      :cImage := "IMoreOptions"
      :Install()
   END

RETURN

//------------------------------------------------------------------------------

CLASS IMoreOptions FROM TIDEPlugin

   METHOD Install()
   METHOD Run( nLang )

   METHOD LoadOptions()
   METHOD SaveOptions()

   METHOD GetInfo()

ENDCLASS

//------------------------------------------------------------------------------

METHOD Install() CLASS IMoreOptions

   LOCAL IEditor  := ::GetEditor()
   LOCAL oMenu := ::GetMainMenu():GetToolsMenu()
   LOCAL oItem
   LOCAL IConfig  := ::GetConfig()
   LOCAL nLang    := IConfig:nLanguage

   ::Register()
   ::LoadOptions()

   IF nLang == 3 .OR. nLang == 10 .OR. nLang == 45 .OR. nLang == 86
      ::cDescription := "Opciones extra del editor"
      oItem := TMenuItem():Create( oMenu, 2, , "&Opciones e&xtra del editor...", , , , "IMoreOptions" )
   ELSE
      oItem := TMenuItem():Create( oMenu, 2, , "&Editor e&xtra options...", , , , "IMoreOptions" )
   ENDIF

   oItem:OnClick := {|| ::Run( nLang ) }
   ::RegisterMenuItem( oItem )

RETURN Nil

//--------------------------------------------------------------------------

METHOD GetInfo() CLASS IMoreOptions

   LOCAL lSpanish := ::nLanguage == 10
   LOCAL cInfo := "<b>MoreOptions</b>" + CRLF + ;
                  CRLF + ;
                  IIF( lSpanish, ;
                     "Aade una nueva opcin 'Opciones extra del editor' al men " + ;
                        "'Herramientas' que permite personalizar opciones del editor " + ;
                        "de cdigo que no son accesibles desde las opciones de configuracin estandar.", ;
                     "Adds a new option 'Editor extra options' to the Tools menu " + ;
                        "to customize some code editor properties which are not part of the standard configuration options." ;
                     ) + CRLF + ;
                  CRLF + ;
                  IIF( lSpanish, ;
                     "<b>Autor</b>:", ;
                     "<b>Author</b>:" ) + " Jos Laln" + CRLF + ;
                  IIF( lSpanish, ;
                     "<b>Versin</b>:", ;
                     "<b>Version</b>:" ) + " " + ::cVersion + CRLF + ;
                  "<b>Copyright</b>: 2011, 2015 Jos Laln & Xailer"

RETURN cInfo

//------------------------------------------------------------------------------

METHOD LoadOptions() CLASS IMoreOptions

   LOCAL IConfig := ::GetConfig()
   LOCAL IEditor := ::GetEditor()

   WITH OBJECT TIni():New( IConfig:cUserPlugins + "\moreoptions.ini" )
      IEditor:SendMsg( SCI_SETCARETWIDTH, :GetEntry( "Editor", "CaretWidth", 2 ) )
      IEditor:SendMsg( SCI_STYLESETBACK, STYLE_BRACELIGHT, :GetEntry( "Editor", "BraceBackColor", IEditor:SendMsg( SCI_STYLEGETBACK, STYLE_BRACELIGHT ) ) )
      IEditor:SendMsg( SCI_SETEXTRAASCENT, :GetNumeric( "Editor", "ExtraAscent", IEditor:SendMsg( SCI_GETEXTRAASCENT ) ) )
      IEditor:SendMsg( SCI_SETEXTRADESCENT, :GetNumeric( "Editor", "ExtraDescent", IEditor:SendMsg( SCI_GETEXTRADESCENT ) ) )

      IEditor:SendMsg( SCI_SETWRAPMODE, :GetNumeric( "Editor", "WrapMode", IEditor:SendMsg( SCI_GETWRAPMODE ) ) )
      IEditor:SendMsg( SCI_SETWRAPVISUALFLAGS, :GetNumeric( "Editor", "WrapModeFlags", IEditor:SendMsg( SCI_GETWRAPVISUALFLAGS ) ) )
      IEditor:SendMsg( SCI_SETWRAPVISUALFLAGSLOCATION, :GetNumeric( "Editor", "WrapModeLocation", IEditor:SendMsg( SCI_GETWRAPVISUALFLAGSLOCATION ) ) )
      IEditor:SendMsg( SCI_SETWRAPINDENTMODE, :GetNumeric( "Editor", "WrapModeIndent", IEditor:SendMsg( SCI_GETWRAPINDENTMODE ) ) )
      IEditor:SendMsg( SCI_SETWRAPSTARTINDENT, :GetNumeric( "Editor", "WrapModeStart", IEditor:SendMsg( SCI_GETWRAPSTARTINDENT ) ) )

      IEditor:SendMsg( SCI_SETMULTIPLESELECTION, :GetLogical( "Editor", "MultiSel", IEditor:SendMsg( SCI_GETMULTIPLESELECTION ) ) )
      IEditor:SendMsg( SCI_SETADDITIONALSELECTIONTYPING, :GetLogical( "Editor", "MultiSelType", IEditor:SendMsg( SCI_GETADDITIONALSELECTIONTYPING ) ) )
      IEditor:SendMsg( SCI_SETMULTIPASTE, :GetLogical( "Editor", "MultiSelPaste", IEditor:SendMsg( SCI_GETMULTIPASTE ) ) )
   END

RETURN Nil

//--------------------------------------------------------------------------

METHOD SaveOptions() CLASS IMoreOptions

   LOCAL IConfig := ::GetConfig()
   LOCAL IEditor := ::GetEditor()
   LOCAL IMenu   := ::GetMainMenu()

   WITH OBJECT TIni():New( IConfig:cXailerBin + "\xailer.cfg" )
      :SetEntry( "Editor", "ViewWhiteSpace", IEditor:nViewWhiteSpace )
      :SetEntry( "Editor", "ViewEOL", IEditor:lViewEOL )
      :SetEntry( "Editor", "CaretClrFore", Hex( IEditor:nCaretClrFore ) )
      :SetEntry( "Editor", "CaretStyle", IEditor:SendMsg( SCI_GETCARETSTYLE ) )
      :SetEntry( "Editor", "FoldStyle", IEditor:nFoldStyle )
      :SetEntry( "Editor", "CompleteBraces", IEditor:lCompleteBraces )
      :SetEntry( "Editor", "CaseMode", IEditor:nCaseMode )
      :SetEntry( "Options", "MenuStyle", IMenu:nStyle )
   END

   WITH OBJECT TIni():New( IConfig:cUserPlugins + "\moreoptions.ini" )
      :SetEntry( "Editor", "CaretWidth", IEditor:SendMsg( SCI_GETCARETWIDTH ) )
      :SetEntry( "Editor", "BraceBackColor", IEditor:SendMsg( SCI_STYLEGETBACK, STYLE_BRACELIGHT ) )
      :SetEntry( "Editor", "ExtraAscent", IEditor:SendMsg( SCI_GETEXTRAASCENT ) )
      :SetEntry( "Editor", "ExtraDescent", IEditor:SendMsg( SCI_GETEXTRADESCENT ) )
      :SetEntry( "Editor", "WrapMode", IEditor:SendMsg( SCI_GETWRAPMODE ) )
      :SetEntry( "Editor", "WrapModeFlags", IEditor:SendMsg( SCI_GETWRAPVISUALFLAGS ) )
      :SetEntry( "Editor", "WrapModeLocation", IEditor:SendMsg( SCI_GETWRAPVISUALFLAGSLOCATION ) )
      :SetEntry( "Editor", "WrapModeIndent", IEditor:SendMsg( SCI_GETWRAPINDENTMODE ) )
      :SetEntry( "Editor", "WrapModeStart", IEditor:SendMsg( SCI_GETWRAPSTARTINDENT ) )
      :SetEntry( "Editor", "MultiSel", IEditor:SendMsg( SCI_GETMULTIPLESELECTION ) )
      :SetEntry( "Editor", "MultiSelType", IEditor:SendMsg( SCI_GETADDITIONALSELECTIONTYPING ) )
      :SetEntry( "Editor", "MultiSelPaste", IEditor:SendMsg( SCI_GETMULTIPASTE ) )
   END

   IF !File( IConfig:cUserPlugins + "\moreoptions.ini" )
      MsgAlert( "Could not create " + IConfig:cUserPlugins + "\moreoptions.ini file." + CRLF + CRLF +;
             "Please check your User plugins dierctory" )
   ENDIF

RETURN Nil

//--------------------------------------------------------------------------

STATIC FUNCTION CaretChooseColor( nClr )

   WITH OBJECT TChooseColorDlg():Create()
      :nColor := nClr
      :lCenter := .T.
      IF :Run()
         nClr := :nColor
      ENDIF
   END

RETURN nClr

//--------------------------------------------------------------------------

STATIC FUNCTION BraceChooseColor( IEditor )

   WITH OBJECT TChooseColorDlg():Create()
      :nColor := IEditor:SendMsg( SCI_STYLEGETBACK, STYLE_BRACELIGHT )
      :lCenter := .T.
      IF :Run()
         IEditor:SendMsg( SCI_STYLESETBACK, STYLE_BRACELIGHT, :nColor )
      ENDIF
   END

RETURN Nil

//--------------------------------------------------------------------------

METHOD Run( nLang ) CLASS IMoreOptions

   LOCAL oForm := ::GetProjectMan():oForm

   WITH OBJECT TFormOptions()
      :IEditor := ::GetEditor()
      :IMenu := ::GetMainMenu()
      :oPlugin := Self
      :nLang := nLang
      :New( oForm )
      :Create()
      :ShowModal()
   END

RETURN Nil

//--------------------------------------------------------------------------
